@@ -10,15 +10,12 @@ ets <- function(y, model="ZZZ", damped=NULL,
   bounds <- match.arg(bounds)
   ic <- match.arg(ic)
 
-  #if(max(y,na.rm=TRUE) > 1e6)
-  #    warning("Very large numbers which may cause numerical problems. Try scaling the data first")
-
   if(any(class(y) %in% c("data.frame","list","matrix","mts")))
     stop("y should be a univariate time series")
   y <- as.ts(y)
 
   # Check if data is constant
-  if (is.constant(y))
+  if (missing(model) & is.constant(y))
     return(ses(y, alpha=0.99999, initial='simple')$model)
 
   # Remove missing values near ends
@@ -68,11 +65,11 @@ ets <- function(y, model="ZZZ", damped=NULL,
       e <- pegelsresid.C(y, m, model$initstate, errortype, trendtype, seasontype, damped, alpha, beta, gamma, phi, nmse)
 
       # Compute error measures
-      np <- length(model$par)
+      np <- length(model$par) + 1
       model$loglik <- -0.5*e$lik
       model$aic <- e$lik + 2*np
       model$bic <- e$lik + log(ny)*np
-      model$aicc <- e$lik + 2*ny*np/(ny-np-1)
+      model$aicc <- model$aic +  2*np*(np+1)/(ny-np-1)
       model$mse <- e$amse[1]
       model$amse <- mean(e$amse)
 
@@ -164,8 +161,15 @@ ets <- function(y, model="ZZZ", damped=NULL,
       stop("Forbidden model combination")
   }
 
-  # Check we have enough data to fit a model
   n <- length(y)
+  # Return non-optimized SES if 4 or  fewer observations
+  if(n <= 4) 
+  {
+    fit <- HoltWintersZZ(orig.y, beta=FALSE, gamma=FALSE, lambda=lambda, biasadj=biasadj)
+    fit$call <- match.call()
+    return(fit)
+  }
+  # Otherwise proceed to check we have enough data to fit a model
   npars <- 2L # alpha + l0
   if(trendtype=="A" | trendtype=="M")
     npars <- npars + 2L # beta + b0
@@ -173,8 +177,8 @@ ets <- function(y, model="ZZZ", damped=NULL,
     npars <- npars + m # gamma + s
   if(!is.null(damped))
     npars <- npars + as.numeric(damped)
-  if(n <= npars + 1)
-    stop("You've got to be joking. I need more data!")
+  if(n <= npars+1)
+    stop("Sorry, but I need more data!")
 
   # Fit model (assuming only one nonseasonal model)
   if(errortype=="Z")
@@ -459,10 +463,11 @@ etsmodel <- function(y, errortype, trendtype, seasontype, damped,
     phi <- fit.par["phi"]
   e <- pegelsresid.C(y,m,init.state,errortype,trendtype,seasontype,damped,alpha,beta,gamma,phi,nmse)
 
-  n <- length(y)
+  np <- np + 1
+  ny <- length(y)
   aic <- e$lik + 2*np
-  bic <- e$lik + log(n)*np
-  aicc <- e$lik + 2*n*np/(n-np-1)
+  bic <- e$lik + log(ny)*np
+  aicc <- aic +  2*np*(np+1)/(ny-np-1)
 
   mse <- e$amse[1]
   amse <- mean(e$amse)
@@ -1016,6 +1021,15 @@ coef.ets <- function(object,...)
   object$par
 }
 
+fitted.ets <- function(object, h=1, ...){
+  if(h==1){
+    return(object$fitted)
+  }
+  else{
+    return(hfitted(object=object, h=h, FUN="ets", ...))
+  }
+}
+
 logLik.ets <- function(object,...)
 {
   structure(object$loglik,df=length(object$par),class="logLik")
