@@ -1,17 +1,17 @@
 # A unit test for arfima.R
-if(require(fpp) & require(testthat))
+if(require(testthat))
 {
-	arfima1 <- arfima(austa, estim = "mle")
-	arfima2 <- arfima(austa, estim = "ls")
-	arfimabc <- arfima(austa, estim = "mle", lambda = 0.75, biasadj=FALSE)
-	arfimabc2 <- arfima(austa, estim = "mle", lambda = 0.75, biasadj=TRUE)
-	
+	arfima1 <- arfima(WWWusage, estim = "mle")
+	arfima2 <- arfima(WWWusage, estim = "ls")
+	arfimabc <- arfima(WWWusage, estim = "mle", lambda = 0.75, biasadj=FALSE)
+	arfimabc2 <- arfima(WWWusage, estim = "mle", lambda = 0.75, biasadj=TRUE)
+
 	test_that("test fitted() and residuals().", {
 		expect_true(all(arimaorder(arfima1) == arimaorder(arfima2)))
 		fitarfima <- fitted(arfima1)
 		residarfima <- residuals(arfima2)
 		expect_true(length(fitarfima) == length(residarfima))
-		expect_true(all(getResponse(arfima1) == austa))
+		expect_true(all(getResponse(arfima1) == WWWusage))
 		expect_false(identical(arfimabc$fitted,arfimabc2$fitted))
 	})
 
